'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

require('core-js/modules/es7.object.values');

require('core-js/modules/es7.object.entries');

require('core-js/modules/es7.object.get-own-property-descriptors');

require('core-js/modules/es7.string.pad-start');

require('core-js/modules/es7.string.pad-end');

var _extend = require('extend');

var _extend2 = _interopRequireDefault(_extend);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _events = require('events');

var _logger = require('./logger');

var _logger2 = _interopRequireDefault(_logger);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

var _template = require('./template');

var _template2 = _interopRequireDefault(_template);

var _environment = require('./environment');

var _environment2 = _interopRequireDefault(_environment);

var _identifier = require('./identifier');

var _identifier2 = _interopRequireDefault(_identifier);

var _rendererror = require('./errors/rendererror');

var _rendererror2 = _interopRequireDefault(_rendererror);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Module with common controller functionality.
 */
/**
 * Copyright 2014 Skytap Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License")
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

class Controller extends _events.EventEmitter {
  constructor() {
    super();
    this.DEFAULT_ERROR_MESSAGE = 'An error occurred. Please try again.';
    this.ERROR_TEMPLATE = 'error';
    this.before = {};
    this.templatePath = '/lib/template';
    this.filtersByRoute = {};
  }

  /**
   * Controller.index(request, response, next)
   * - request (Object): Express request object
   * - response (Object): Express response object
   * - next (Function): Express function to invoke the next handler
   *
   * GET /resource
   */
  index(request, response, next) {
    next();
  }

  /**
   * Controller.new(request, response, next)
   * - request (Object): Express request object
   * - response (Object): Express response object
   * - next (Function): Express function to invoke the next handler
   *
   * GET /resource/new
   */
  new(request, response, next) {
    next();
  }

  /**
   * Controller.create(request, response, next)
   * - request (Object): Express request object
   * - response (Object): Express response object
   * - next (Function): Express function to invoke the next handler
   *
   * POST /resource
   */
  create(request, response, next) {
    next();
  }

  /**
   * Controller.show(request, response, next)
   * - request (Object): Express request object
   * - response (Object): Express response object
   * - next (Function): Express function to invoke the next handler
   *
   * GET /resource/:id
   */
  show(request, response, next) {
    next();
  }

  /**
   * Controller.edit(request, response, next)
   * - request (Object): Express request object
   * - response (Object): Express response object
   * - next (Function): Express function to invoke the next handler
   *
   * GET /resource/:id/edit
   */
  edit(request, response, next) {
    next();
  }

  /**
   * Controller.update(request, response, next)
   * - request (Object): Express request object
   * - response (Object): Express response object
   * - next (Function): Express function to invoke the next handler
   *
   * PUT /resource/:id
   */
  update(request, response, next) {
    next();
  }

  /**
   * Controller.destroy(request, response, next)
   * - request (Object): Express request object
   * - response (Object): Express response object
   * - next (Function): Express function to invoke the next handler
   *
   * DELETE /resource/:id
   */
  destroy(request, response, next) {
    next();
  }

  /**
   * Controller.addFiltersForHandler(url, handler)
   * - url (String)
   * - handler (String)
   */
  addFiltersForHandler(url, handler) {
    if (!{}.hasOwnProperty.call(this.filtersByRoute, url)) {
      this.filtersByRoute[url] = {};
    }

    this.filtersByRoute[url][handler] = [];

    if (typeof this.before === 'object') {
      Object.entries(this.before).forEach(([filter, handlers]) => {
        if (handlers.indexOf(handler) !== -1 || handlers[0] === 'all') {
          this.filtersByRoute[url][handler].push(filter);
        }
      });
    }
  }

  /**
   * Controller.getFilters(url, handler) -> Array
   * - url (String)
   * - handler (String)
   */
  getFilters(url, handler) {
    return this.filtersByRoute[url][handler];
  }

  /**
   * Controller.render(request, response, templateName, templateValues, send) -> Promise
   * - request (Object): Express request object
   * - response (Object): Express response object
   * - templateName (String)
   * - templateValues (Object)
   * - send (Boolean)
   */
  render(request, response, templateName, templateValues, send) {
    const defaults = {
      layout: 'layouts/application',
      title: '',
      stylesheets: [],
      response,
      request,
      flash: {},
      Config: _config2.default,
      _content: {}
    };

    return new _bluebird2.default((resolve, reject) => {
      const allTemplateValues = (0, _extend2.default)(defaults, templateValues);

      Object.entries(_template2.default.getMixins()).forEach(([name, Helper]) => {
        allTemplateValues[name] = Helper.bind(allTemplateValues);
      });

      Object.entries(allTemplateValues).forEach(([key, value]) => {
        response.locals[key] = value;
      });

      const startTime = Date.now();

      response.on('finish', () => {
        this.emit('render-finished', this, request, startTime, templateName);
        this.requestFinished(request);
      });

      response.render(templateName, { layout: allTemplateValues.layout }, (error, result) => {
        _logger2.default.debug(`${request.method} ${request.url} DONE`, request);

        _logger2.default.info(`Request for ${request.minorjs.controller.name}#${request.minorjs.controller.action} done in ${Date.now() - request.minorjs.start} ms`, request);

        if (error) {
          this.handleRenderError(request, response, error, templateName);
          reject(new _rendererror2.default());
        } else {
          const shouldSend = typeof send === 'undefined' ? true : send;
          if (shouldSend) {
            response.send(result);
          }
          resolve(result);
        }
      });
    });
  }

  /**
   * Controller.getFullError() -> String
   */
  getFullError(request, response, error, incidentId) {
    return `Incident ${incidentId} - ${error.stack}`;
  }

  /**
   * Controller.getUserError() -> String
   */
  getUserError(request, response, error, incidentId) {
    return `${this.DEFAULT_ERROR_MESSAGE} (${incidentId})`;
  }

  /**
   * Controller.handleError(request, response, error)
   */
  handleError(request, response, error) {
    if (error instanceof _rendererror2.default) {
      // error was already handled
      return;
    }

    const incidentId = _identifier2.default.generate();
    const fullError = this.getFullError(request, response, error, incidentId);

    _logger2.default.error(fullError, request);

    if (request.xhr) {
      this.handleXhrError(request, response, error);
    } else {
      this.handleNormalError(request, response, error, incidentId);
    }
  }

  /**
   * Controller.handleXhrError(request, response, error)
   */
  handleXhrError(request, response, error) {
    response.status(500).send({
      success: false,
      error
    });
  }

  /**
   * Controller.handleNormalError(request, response, error, incidentId)
   */
  handleNormalError(request, response, error, incidentId) {
    const userError = this.getUserError(request, response, error, incidentId);

    response.status(500);

    this.render(request, response, this.ERROR_TEMPLATE, {
      userError,
      error: _environment2.default.isDevelopment() ? error : null,
      layout: 'layouts/error'
    });
  }

  /**
   * Controller.handleRenderError(request, response, error, templateName)
   * - request (Object): Request object.
   * - response (Object): Response object.
   * - error (Object)
   * - templateName (String)
   */
  handleRenderError(request, response, error, templateName) {
    if (templateName === this.ERROR_TEMPLATE) {
      const incidentId = _identifier2.default.generate();
      const fullError = this.getFullError(request, response, error, incidentId);
      const userError = this.getUserError(request, response, error, incidentId);
      _logger2.default.error(`Could not render error page. ${fullError}`, request);
      response.send(userError);
    } else {
      this.handleError(request, response, error);
    }
  }

  /**
   * Controller.requestFinished(request)
   * - request (Object): Request object.
   */
  requestFinished(request) {
    const results = (0, _extend2.default)({
      method: request.method,
      time: Date.now() - request.minorjs.start
    }, request.minorjs);
    this.emit('request-finished', request, results);
  }
}
exports.default = Controller;