'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

require('core-js/modules/es7.object.values');

require('core-js/modules/es7.object.entries');

require('core-js/modules/es7.object.get-own-property-descriptors');

require('core-js/modules/es7.string.pad-start');

require('core-js/modules/es7.string.pad-end');

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _filesystem = require('./filesystem');

var _filesystem2 = _interopRequireDefault(_filesystem);

var _logger = require('./logger');

var _logger2 = _interopRequireDefault(_logger);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Module to load and manage controller filters.
 */
/**
 * Copyright 2014 Skytap Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License")
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

const Filter = {

  filters: {},

  /**
   * Filter.getFilters() -> Object
   */
  getFilters() {
    return this.filters;
  },

  /**
   * Filter.load(path) -> Object
   * - path (String): Path to the filters
   */
  load(path) {
    const start = Date.now();

    return _filesystem2.default.requireFilesInDirectory(path, true).then(filters => {
      this.filters = filters;
      _logger2.default.profile('Load filters', start);
    });
  },

  /**
   * Filter.run(filters, request, response, next) -> Boolean
   * - filters (Array): All filters to run
   * - request (Object): Express request object
   * - response (Object): Express response object
   * - next (Function): Express next callback
   */
  run(filters, request, response, next) {
    const filterFunctions = [];

    if (!(Array.isArray(filters) && filters.length)) {
      return _bluebird2.default.resolve();
    }

    filters.forEach(filter => {
      if ({}.hasOwnProperty.call(this.filters, filter)) {
        filterFunctions.push(this.runFilter(filter, request, response, next));
      }
    });

    // run each filter in sequence
    return filterFunctions.reduce((currentValue, nextValue) => currentValue.then(nextValue), _bluebird2.default.resolve());
  },

  /**
   * Filter.runFilter(filter, request, response, next) -> Function
   * - filter (String): Filter name
   * - request (Object): Express request object
   * - response (Object): Express response object
   * - next (Function): Express next callback
   */
  runFilter(filter, request, response, next) {
    return () => {
      try {
        _logger2.default.debug(`Running the ${this.filters[filter].moduleName} filter`, request);
        const result = this.filters[filter].process(request, response, next);
        return result instanceof _bluebird2.default ? result : _bluebird2.default.resolve(result);
      } catch (e) {
        _logger2.default.debug(`Rejected ${this.filters[filter].moduleName} filter`, request);
        return _bluebird2.default.reject(e);
      }
    };
  }
};

exports.default = Filter;